<?php

namespace FiloBlu\Flow\Model\Parser;

/**
 * Class AbstractParser
 * @package FiloBlu\Flow\Model\Parser
 */
abstract class AbstractParser
{
    /**
     * @var \FiloBlu\Flow\Model\Map
     */
    public $map;
    /**
     * @var
     */
    protected $_channel;
    /**
     * @var \Monolog\Logger
     */
    protected $_logger;
    private $attribute_set_id;
    private $configurable_attributes;
    private $websites;
    private $stores;

    /**
     * AbstractParser constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
    ) {
        $this->_logger = $loggerProvider->getLogger();

    }

    /**
     * @param $resource
     * @return mixed
     */
    abstract public function process($resource);

    /**
     * @param $config
     * @return $this
     */
    public function setConfig($config)
    {
        return $this;
    }

    /**
     * @param $maplocation
     * @return bool
     */
    public function loadMap($maplocation)
    {
        /** @var \FiloBlu\Flow\Model\Map $map */
        $map = \Magento\Framework\App\ObjectManager::getInstance()->create(\FiloBlu\Flow\Model\Map::class);
        $map->load($maplocation);
        if ($map->initialized) {
            $this->map = $map;
            return true;
        }
        return false;
    }

    /**
     * Parse Data
     *
     * @param $data
     * @param bool $key_mode
     * @throws \Exception
     */
    public function parse($data, $key_mode = false)
    {
        if (!$key_mode && !$this->map->checkData($data)) {
            throw new \Exception('Data don\'t match mapping count ' . count($data));
        }

        $channel = $this->getChannel();

        $fields = $this->map->getFields();
        $parsed_data = [];

        foreach ($fields as $i => $field) {
            $validators = [];
            $filters = [];
            $fieldMap = $this->map->get($field);
            $key = ($key_mode) ? (isset($fieldMap['from']) ? $fieldMap['from'] : $i) : $i;
            if (array_key_exists(trim($key), $data)) {
                $fieldData = $data[trim($key)];
                $value = null;

                if (isset($fieldMap['skip'])) {
                    continue;
                }

                if (isset($fieldMap['validators'])) {
                    $validators = $fieldMap['validators'];
                }

                if (isset($fieldMap['filters'])) {
                    $filters = $fieldMap['filters'];
                }

                if ($this->_validate($fieldData, $validators, $field)) {
                    $value = $this->_filter($fieldData, $filters, $data);
                    $parsed_data[$field] = $value;
                } else {
                    throw new \Exception("{$value} is not Valid for {$field}");
                }

            }

        }

        if (method_exists($channel, 'insertData') && !is_null($channel)) {

            try {
                $channel->insertData($parsed_data);
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
            }

        } else {
            throw new \Exception('No method insertData for class ' . get_class($channel));
        }

    }

    /**
     * @return mixed
     */
    public function getChannel()
    {
        return $this->_channel;
    }

    /**
     * @param $channel
     */
    public function setChannel($channel)
    {
        $this->_channel = $channel;
    }

    /**
     * Validate single field applying Validators
     *
     * @param $value
     * @param $validators
     * @param $field
     * @return bool
     */
    protected function _validate($value, $validators, $field)
    {
        foreach ($validators as $validator) {
            $validatorClass = 'FiloBlu\Bouncer\Model\Parser\Validators' . "\\" . ucfirst($validator);
            $validator = new $validatorClass();
            if (!$validator->isValid($value)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Validate single field applying Validators
     *
     * @param $value
     * @param $filters
     * @param $data
     * @return mixed
     */
    protected function _filter($value, $filters, $data)
    {
        if (count($filters) > 0) {
            foreach ($filters as $filterData) {
                $filterClass = 'FiloBlu\Flow\Model\Filters\\' . ucfirst($filterData['class']);
                $filter = \Magento\Framework\App\ObjectManager::getInstance()->create($filterClass);
                if (method_exists($filter, 'setRow')) {
                    $filter->setRow($data);
                }
                $value = $filter->filter($value, $filterData['args']);
            }
        }
        return $value;
    }
}
